unit DataEntryForm;

interface

uses
  SysUtils, Types, UITypes, Classes, Variants, FMX_Types, FMX_Controls, FMX_Forms,
  FMX_Dialogs, FMX_ExtCtrls, FMX_Edit, FMX_Platform;

type
  TfrmDataEntry = class(TForm)
    Panel1: TPanel;
    btnCancel: TCornerButton;
    edtFirstName: TEdit;
    lblFirstName: TLabel;
    edtLastName: TEdit;
    lblLastName: TLabel;
    clndrbxDOB: TCalendarBox;
    btnSave: TCornerButton;
    Warning1: TImageControl;
    Warning2: TImageControl;
    StyleBook1: TStyleBook;
    procedure edtFirstNameEnter(Sender: TObject);
    procedure edtFirstNameExit(Sender: TObject);
    procedure edtLastNameEnter(Sender: TObject);
    procedure edtLastNameExit(Sender: TObject);
    procedure Panel1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure AnyEditChange(Sender: TObject);
  private
    function StateOkay: boolean;
    function GetDOB: TDateTime;
    function GetFirstName: string;
    function GetLastName: string;
    procedure SetDOB(const Value: TDateTime);
    procedure SetFirstName(const Value: string);
    procedure SetLastName(const Value: string);
  public
    procedure ClearFieldLabels;
    property FirstName: string read GetFirstName write SetFirstName;
    property LastName: string read GetLastName write SetLastName;
    property DOB: TDateTime read GetDOB write SetDOB;
  end;

var
  frmDataEntry: TfrmDataEntry;

implementation

{$R *.lfm}

uses DataStorageMain;

procedure TfrmDataEntry.SetDOB(const Value: TDateTime);
begin
  clndrbxDOB.Date := Value;
end;

procedure TfrmDataEntry.SetFirstName(const Value: string);
begin
  edtFirstName.Text := Value;
end;

procedure TfrmDataEntry.SetLastName(const Value: string);
begin
  edtLastName.Text := Value;
end;

function TfrmDataEntry.StateOkay: boolean;
begin
  result := (edtFirstName.Text <> '') and (edtLastName.Text <> '');
  Warning1.Visible := edtFirstName.Text = '';
  Warning2.Visible := edtLastName.Text = '';
end;

procedure TfrmDataEntry.AnyEditChange(Sender: TObject);
begin
  Warning1.Visible := False;
  Warning2.Visible := False;
end;

procedure TfrmDataEntry.btnSaveClick(Sender: TObject);
begin
  if StateOkay then
    ModalResult := mrOK;
end;

procedure TfrmDataEntry.ClearFieldLabels;
begin
  lblFirstName.Visible := false;
  lblLastName.Visible := false;
end;

procedure TfrmDataEntry.edtFirstNameEnter(Sender: TObject);
begin
  lblFirstName.Visible := false;
end;

procedure TfrmDataEntry.edtFirstNameExit(Sender: TObject);
begin
  if edtFirstName.Text = '' then
    lblFirstName.Visible := true;
end;

procedure TfrmDataEntry.edtLastNameEnter(Sender: TObject);
begin
  lblLastName.Visible := false;
end;

procedure TfrmDataEntry.edtLastNameExit(Sender: TObject);
begin
  if edtLastName.Text = '' then
    lblLastName.Visible := true;
end;

procedure TfrmDataEntry.FormCreate(Sender: TObject);
begin
  edtFirstName.MaxLength := 25;
  edtLastName.MaxLength := 25;
  StyleBook := frmDataStorage.StyleBook2;
end;

function TfrmDataEntry.GetDOB: TDateTime;
begin
  result := clndrbxDOB.Date;
end;

function TfrmDataEntry.GetFirstName: string;
begin
  result := edtFirstName.Text;
end;

function TfrmDataEntry.GetLastName: string;
begin
  result := edtLastName.Text;
end;

procedure TfrmDataEntry.Panel1Click(Sender: TObject);
begin
  Platform.HideVirtualKeyboard;
end;

end.
