unit DataStorageMain;

{$IFDEF FPC}
  {$mode objfpc}{$H+}
  {$modeswitch objectivec1}
{$ENDIF}

interface

uses
  SysUtils, Types, UITypes, Classes, Variants, FMX_Types, FMX_Controls, FMX_Forms,
  FMX_Dialogs, FMX_ExtCtrls, FMX_Layouts, FMX_Grid, FMX_Memo
{$IFDEF FPC}
  , iPhoneAll, Sqlite3DS, SQLite3conn
{$ENDIF}
  ;

type
  TfrmDataStorage = class(TForm)
    Layout1: TLayout;
    btnAdd: TCornerButton;
    btnEdit: TCornerButton;
    btnDelete: TCornerButton;
    grdData: TStringGrid;
    StringColumn1: TStringColumn;
    StringColumn2: TStringColumn;
    StringColumn3: TStringColumn;
    StyleBook2: TStyleBook;
    StringColumn4: TStringColumn;
    procedure btnAddClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnEditClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    procedure RemoveRow;
  public
{$IFDEF FPC}
    procedure LoadData;
    function GetDocumentsPath: string;
    procedure AddRowToTable(FirstName, LastName: string; DOB: TDateTime);
    procedure UpdateRowInTable(FirstName, LastName: string; DOB: TDateTime);
    procedure DeleteRowFromTable(ID: integer);
    procedure ConfigureDatabaseAndTable;
{$ENDIF}
  end;

var
  frmDataStorage: TfrmDataStorage;

implementation

uses DataEntryForm;

const
  DATATABLENAME: string = 'PersonData';
  DBFILENAME: string = 'DataStorage.sqlite';

type
  TDataRec = record
    FirstName: string[20];
    LastName: string[20];
    DateOfBirth: TDateTime;
  end;

{$R *.lfm}

procedure TfrmDataStorage.btnAddClick(Sender: TObject);
var
  IDX: integer;
begin
  frmDataEntry.Top := 2;
  frmDataEntry.Left := (ClientWidth - frmDataEntry.ClientWidth) div 2;
  frmDataEntry.edtFirstName.Text := '';
  frmDataEntry.edtLastName.Text := '';
  if frmDataEntry.ShowModal = mrOK then
  begin
    grdData.RowCount := grdData.RowCount + 1;
    IDX := grdData.RowCount - 1;
    grdData.Cells[0, IDX] := frmDataEntry.FirstName;
    grdData.Cells[1, IDX] := frmDataEntry.LastName;
    grdData.Cells[2, IDX] := DateToStr(frmDataEntry.DOB);
    grdData.Selected := IDX;

    {$IFDEF FPC}
      AddRowToTable(frmDataEntry.FirstName, frmDataEntry.LastName,
        frmDataEntry.DOB);
    {$ENDIF}
  end;
end;

procedure TfrmDataStorage.btnEditClick(Sender: TObject);
var
  IDX: integer;
begin
  frmDataEntry.Top := 2;
  frmDataEntry.Left := (ClientWidth - frmDataEntry.ClientWidth) div 2;
  IDX := grdData.Selected;
  frmDataEntry.FirstName := grdData.Cells[0, IDX];
  frmDataEntry.LastName := grdData.Cells[1, IDX];
  frmDataEntry.DOB := StrToDate(grdData.Cells[2, IDX]);
  frmDataEntry.ClearFieldLabels;

  if frmDataEntry.ShowModal = mrOK then
  begin
    grdData.Cells[0, IDX] := frmDataEntry.FirstName;
    grdData.Cells[1, IDX] := frmDataEntry.LastName;
    grdData.Cells[2, IDX] := DateToStr(frmDataEntry.DOB);

    {$IFDEF FPC}
      UpdateRowInTable(frmDataEntry.FirstName, frmDataEntry.LastName,
        frmDataEntry.DOB);
    {$ENDIF}
  end;
end;

procedure TfrmDataStorage.btnDeleteClick(Sender: TObject);
var
  ID, IDX: integer;
begin
  if grdData.RowCount = 0 then
    exit;

  if MessageDlg('Delete row?', TMsgDlgType.mtConfirmation,
    [TMsgDlgBtn.mbYes, TMsgDlgBtn.mbNo], 0) = mrYes then
  begin
    IDX := grdData.Selected;
    ID := StrToInt(grdData.Cells[3, IDX]);
    RemoveRow;
    {$IFDEF FPC}
    DeleteRowFromTable(ID);
    {$ENDIF}
  end;
end;

procedure TfrmDataStorage.RemoveRow;
var
  IDX, I: integer;
begin
  IDX := grdData.Selected;
  for I := IDX to grdData.RowCount - 2 do
  begin
    grdData.Cells[0, I] := grdData.Cells[0, I + 1];
    grdData.Cells[1, I] := grdData.Cells[1, I + 1];
    grdData.Cells[2, I] := grdData.Cells[2, I + 1];
  end;
  grdData.RowCount := grdData.RowCount - 1;
end;

procedure TfrmDataStorage.FormCreate(Sender: TObject);
begin
{$IFDEF FPC}
  LoadData;
{$ENDIF}
end;


{$IFDEF FPC}
function TfrmDataStorage.GetDocumentsPath: string;
var
  HomePath: string;
begin
  HomePath := NSHomeDirectory.UTF8String;
  result := IncludeTrailingBackslash(HomePath) + 'Documents/';
end;

procedure TfrmDataStorage.ConfigureDatabaseAndTable;
var
  Conn: TSQLite3Connection;
  FilePath, SQL: string;
  DS: TSqlite3Dataset;
begin
  FilePath := GetDocumentsPath + DBFILENAME;
  // Ensure the database exists
  Conn := TSQLite3Connection.Create(nil);
  Conn.DatabaseName := FilePath;
  Conn.Connected := true;
  Conn.Free;

  // Check that the table exists
  DS := TSqlite3Dataset.Create(nil);
  DS.FileName := FilePath;
  if not DS.TableExists(DATATABLENAME) then
  begin
    SQL := 'create table ' + DATATABLENAME
      + '(ID AutoInc, FirstName char(25), LastName char(25), DOB date)';
      DS.ExecSQL(SQL);
  end;

  DS.Free
end;

procedure TfrmDataStorage.LoadData;
var
  FilePath: string;
  IDX: integer;
  DS: TSqlite3Dataset;
begin
  ConfigureDatabaseAndTable;
  FilePath := GetDocumentsPath + DBFILENAME;

  DS := TSqlite3Dataset.Create(nil);
  DS.FileName := FilePath;
  DS.TableName := DATATABLENAME;
  DS.Open;

  while not DS.EOF do
  begin
    grdData.RowCount := grdData.RowCount + 1;
    IDX := grdData.RowCount - 1;
    grdData.Cells[0, IDX] := DS.FieldByName('FirstName').AsString;
    grdData.Cells[1, IDX] := DS.FieldByName('LastName').AsString;
    grdData.Cells[2, IDX] := DS.FieldByName('DOB').AsString;
    grdData.Cells[3, IDX] := DS.FieldByName('ID').AsString;
    DS.Next;
  end;

  DS.Free;
  grdData.Selected := 0;
end;

procedure TfrmDataStorage.AddRowToTable(FirstName, LastName: string; DOB: TDateTime);
var
  FilePath: string;
  DS: TSqlite3Dataset;
  ID: integer;
begin
  FilePath := GetDocumentsPath + DBFILENAME;

  DS := TSqlite3Dataset.Create(nil);
  DS.FileName := FilePath;
  DS.TableName := DATATABLENAME;
  DS.PrimaryKey := 'ID';
  DS.Open;

  DS.Insert;
  DS.FieldByName('FirstName').AsString := FirstName;
  DS.FieldByName('LastName').AsString := LastName;
  DS.FieldByName('DOB').AsDateTime := DOB;
  DS.Post;
  DS.ApplyUpdates;
  ID := DS.LastInsertRowID;
  DS.Free;

  // Store the ID in the hidden column
  grdData.Cells[3, grdData.Selected] := IntToStr(ID);
end;

procedure TfrmDataStorage.UpdateRowInTable(FirstName, LastName: string; DOB: TDateTime);
var
  FilePath: string;
  DS: TSqlite3Dataset;
  ID, IDX: integer;
begin
  FilePath := GetDocumentsPath + DBFILENAME;

  IDX := grdData.Selected;
  ID := StrToInt(grdData.Cells[3, IDX]);

  DS := TSqlite3Dataset.Create(nil);
  DS.FileName := FilePath;
  DS.TableName := DATATABLENAME;
  DS.PrimaryKey := 'ID';
  DS.Open;

  if DS.Locate('ID', ID, []) then
  begin
    DS.Edit;
    DS.FieldByName('FirstName').AsString := FirstName;
    DS.FieldByName('LastName').AsString := LastName;
    DS.FieldByName('DOB').AsDateTime := DOB;
    DS.Post;
    DS.ApplyUpdates;
  end;
  DS.Free;
end;

procedure TfrmDataStorage.DeleteRowFromTable(ID: integer);
var
  FilePath: string;
  DS: TSqlite3Dataset;
  SQL: string;
begin
  FilePath := GetDocumentsPath + DBFILENAME;
  DS := TSqlite3Dataset.Create(nil);
  DS.FileName := FilePath;
  SQL := format('delete from %s where ID = %d', [DATATABLENAME, ID]);
  DS.ExecSQL(SQL);
  DS.Free;
end;

{$ENDIF}

end.
